"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseComponent = exports.createAbellComponentContext = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const general_utils_1 = require("../utils/general-utils");
const hash_1 = __importDefault(require("../utils/hash"));
const compiler_1 = require("../compiler");
const css_parser_1 = require("./css-parser");
function parseAttributes(attrString) {
    const attributeMatches = attrString.match(/(?:[^\s"']+|(["'])[^"]*\1)+/g);
    if (!attributeMatches) {
        return {};
    }
    return attributeMatches.reduce((prevObj, val) => {
        const firstEqual = val.indexOf('=');
        if (firstEqual < 0) {
            return Object.assign(Object.assign({}, prevObj), { [val]: true });
        }
        const key = val.slice(0, firstEqual);
        let value = val.slice(firstEqual + 1).trim();
        if ((value.startsWith('"') && value.endsWith('"')) ||
            (value.startsWith("'") && value.endsWith("'"))) {
            value = value.slice(1, -1);
        }
        return Object.assign(Object.assign({}, prevObj), { [key]: value });
    }, {});
}
function getStyleScriptMatches(abellComponentPath, abellComponentContent, componentHash) {
    const styleMatches = general_utils_1.execRegexOnAll(/\<style(.*?)\>(.*?)\<\/style\>/gs, abellComponentContent).matches.map((contentMatch) => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [_, blockAttributes, blockContent] = contentMatch;
        const attributes = parseAttributes(blockAttributes);
        let content;
        if (!false) {
            // if it is css then scope it by appending hash to css selector
            content = css_parser_1.cssSerializer(blockContent, componentHash);
        }
        else {
            content = blockContent;
        }
        return {
            component: path_1.default.basename(abellComponentPath),
            componentPath: abellComponentPath,
            content,
            attributes
        };
    });
    const scriptMatches = general_utils_1.execRegexOnAll(/\<script(.*?)\>(.*?)\<\/script\>/gs, abellComponentContent).matches.map((contentMatch) => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const [_, blockAttributes, blockContent] = contentMatch;
        const attributes = parseAttributes(blockAttributes);
        let content;
        if (blockContent.includes('scopedSelector')) {
            // if it is javascript then scope it by injecting scopedSelector functions
            content = blockContent.replace(/scoped(Selector|SelectorAll)\((.*?)\)/g, `document.query$1($2 + "[data-abell-${componentHash}]")`);
        }
        else {
            content = blockContent;
        }
        return {
            component: path_1.default.basename(abellComponentPath),
            componentPath: abellComponentPath,
            content,
            attributes
        };
    });
    return { styleMatches, scriptMatches };
}
function createAbellComponentContext(abellComponentPath, options) {
    // Runs when the abell component require is called
    const basePath = path_1.default.dirname(abellComponentPath);
    const filename = path_1.default.relative(process.cwd(), abellComponentPath);
    const newOptions = Object.assign(Object.assign({}, options), { filename,
        basePath });
    let abellComponentContent = fs_1.default.readFileSync(abellComponentPath, 'utf8');
    // Add styles and scripts to component bundle map
    // we use the relative path here so that hash doesn't change across machines
    const componentHash = hash_1.default(general_utils_1.normalizePath(path_1.default.relative(process.cwd(), abellComponentPath)));
    const { scriptMatches, styleMatches } = getStyleScriptMatches(abellComponentPath, abellComponentContent, componentHash);
    const isStyleGlobal = styleMatches.length <= 0 ||
        styleMatches.every((styleMatch) => styleMatch.attributes.global === true);
    if (options && !isStyleGlobal) {
        // ignore adding scope hash
        // TODO: only prefix HTML tags inside the template tagx
        abellComponentContent = general_utils_1.prefixHtmlTags(abellComponentContent, componentHash);
    }
    const componentBundleMap = {
        component: path_1.default.basename(filename),
        filepath: filename,
        scripts: scriptMatches,
        styles: styleMatches,
        components: []
    };
    return {
        // The function that gets called when we do <Hello props={xyz: 'hi'} />
        AbellComponentCall: (props) => {
            const componentMap = parseComponent(abellComponentContent, props, newOptions);
            // populate component bundle map with information
            componentBundleMap.components = componentMap.components;
            return componentMap;
        },
        getComponentBundleMap: () => componentBundleMap
    };
}
exports.createAbellComponentContext = createAbellComponentContext;
function parseComponent(abellComponentContent, props, options) {
    var _a;
    const { builtInFunctions, subComponents } = general_utils_1.getAbellInBuiltSandbox(options);
    const sandbox = Object.assign({ props }, builtInFunctions);
    const compiledComponentContent = compiler_1.compile(abellComponentContent, sandbox, options);
    const templateTag = compiledComponentContent.match(/\<template.*?>(.*?)\<\/template>/s // TODO: remove .*? from template tag
    );
    return {
        html: (_a = templateTag === null || templateTag === void 0 ? void 0 : templateTag[1]) !== null && _a !== void 0 ? _a : '',
        components: subComponents
    };
}
exports.parseComponent = parseComponent;
/**
 * TODO:
 * - Add typescript support to css-parser, and hash
 * -
 *
 */
