"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizePath = exports.prefixHtmlTags = exports.copyFolderSync = exports.createPathIfAbsent = exports.isComponentImported = exports.execRegexOnAll = exports.getAbellInBuiltSandbox = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const abell_component_parser_1 = require("../parsers/abell-component-parser");
const ABELL_CSS_DATA_PREFIX = 'data-abell';
function getAbellInBuiltSandbox(options) {
    const subComponents = [];
    const builtInFunctions = {
        console: {
            log: (message) => console.log(options.filename ? colors.gray(`[${options.filename}]\n`) : undefined, message)
        },
        __filename: options.filename ? path_1.default.resolve(options.filename) : undefined,
        __dirname: options.basePath ? path_1.default.resolve(options.basePath) : undefined
    };
    if (options.allowRequire) {
        builtInFunctions.require = (pathToRequire) => {
            var _a;
            const fullRequirePath = path_1.default.join((_a = options.basePath) !== null && _a !== void 0 ? _a : '', pathToRequire);
            if (fullRequirePath.endsWith('.abell')) {
                // TODO:
                const { AbellComponentCall, getComponentBundleMap } = abell_component_parser_1.createAbellComponentContext(fullRequirePath, options);
                subComponents.push(getComponentBundleMap());
                return AbellComponentCall;
            }
            if (fs_1.default.existsSync(fullRequirePath)) {
                // Local file require
                return require(fullRequirePath);
            }
            try {
                // NPM Package or NodeJS Module
                return require(pathToRequire);
            }
            catch (err) {
                if (err.code === 'MODULE_NOT_FOUND') {
                    // throw custom error here.
                    throw new Error('MODULE_NOT_FOUND');
                }
                else {
                    throw err;
                }
            }
        };
    }
    return { builtInFunctions, subComponents };
}
exports.getAbellInBuiltSandbox = getAbellInBuiltSandbox;
const colors = {
    red: (message) => `\u001b[31m${message}\u001b[39m`,
    yellow: (message) => `\u001b[1m\u001b[33m${message}\u001b[39m\u001b[22m`,
    gray: (message) => `\u001b[90m${message}\u001b[39m`
};
const execRegexOnAll = (regex, template) => {
    /** allMatches holds all the results of RegExp.exec() */
    const allMatches = [];
    let match = regex.exec(template);
    if (!match) {
        return { matches: [], input: template };
    }
    const { input } = match;
    while (match !== null) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        delete match.input;
        allMatches.push(match);
        match = regex.exec(template);
    }
    return { matches: allMatches, input };
};
exports.execRegexOnAll = execRegexOnAll;
const isComponentImported = (abellCode) => /(?:const|var|let) (\w*) *?= *?require\(["'`](.*?)\.abell["'`]\)/g.test(abellCode);
exports.isComponentImported = isComponentImported;
const createPathIfAbsent = (pathToCreate) => {
    // prettier-ignore
    pathToCreate
        .split(path_1.default.sep)
        .reduce((prevPath, folder) => {
        const currentPath = path_1.default.join(prevPath, folder, path_1.default.sep);
        if (!fs_1.default.existsSync(currentPath)) {
            fs_1.default.mkdirSync(currentPath);
        }
        return currentPath;
    }, '');
};
exports.createPathIfAbsent = createPathIfAbsent;
const copyFolderSync = (from, to, ignore = [], ignoreEmptyDirs = true) => {
    if (ignore.includes(from)) {
        return;
    }
    const fromDirectories = fs_1.default.readdirSync(from);
    exports.createPathIfAbsent(to);
    fromDirectories.forEach((element) => {
        const fromElement = path_1.default.join(from, element);
        const toElement = path_1.default.join(to, element);
        if (fs_1.default.lstatSync(fromElement).isFile()) {
            if (!ignore.includes(fromElement)) {
                fs_1.default.copyFileSync(fromElement, toElement);
            }
        }
        else {
            exports.copyFolderSync(fromElement, toElement, ignore);
            if (fs_1.default.existsSync(toElement) && ignoreEmptyDirs) {
                try {
                    fs_1.default.rmdirSync(toElement);
                }
                catch (err) {
                    if (err.code !== 'ENOTEMPTY')
                        throw err;
                }
            }
        }
    });
};
exports.copyFolderSync = copyFolderSync;
// This function uses a single regular expression to add a data prefix to every html opening tag passed to it
const prefixHtmlTags = (htmlString, hash) => {
    const openingTagRegexp = /\<([a-zA-Z]+)(.*?)(\s?\/?)\>/gs;
    return htmlString.replace(openingTagRegexp, `<$1$2 ${ABELL_CSS_DATA_PREFIX}-${hash}$3>`);
};
exports.prefixHtmlTags = prefixHtmlTags;
const normalizePath = (path) => {
    const isExtendedLengthPath = /^\\\\\?\\/.test(path);
    const hasNonAscii = /[^\u0000-\u0080]+/.test(path); // eslint-disable-line no-control-regex
    if (isExtendedLengthPath || hasNonAscii) {
        return path;
    }
    return path.replace(/\\/g, '/');
};
exports.normalizePath = normalizePath;
